/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.mediatypes.builtin;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.RegistryException;
import org.wso2.registry.Resource;
import org.wso2.registry.User;
import org.wso2.registry.i18n.Messages;
import org.wso2.registry.jdbc.mediatypes.MediaTypeHandler;
import org.wso2.registry.jdbc.mediatypes.MediaTypeManager;
import org.wso2.registry.utils.AuthorizationUtil;
import org.wso2.usermanager.Realm;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Iterator;
import java.util.List;

public class DefaultMediaTypeHandler extends MediaTypeHandler {

    private static final Log log = LogFactory.getLog(DefaultMediaTypeHandler.class);

    public DefaultMediaTypeHandler(DataSource dataSource,
                                   Realm realm,
                                   MediaTypeManager mediaTypeManager) {

        super(dataSource, realm, mediaTypeManager);
    }

    public Resource get(String path, Resource resource) throws RegistryException {

        String plainPath = path;
        long versionNumber = -1;
        if (path.indexOf("?v=") != -1) {
            String[] parts = path.split("\\?v=");
            plainPath = parts[0];
            versionNumber = Long.parseLong(parts[1]);
        }

        Connection conn = null;
        try {
            conn = dataSource.getConnection();
        } catch (SQLException e) {
            throw new RegistryException(e.getMessage());
        }
        //if we uncomment following three lines then , no onc access a deleted resource
//        if (!resourceDAO.isResourceActive(plainPath,conn)) {
//             throw new RegistryException("Trying to access a deleted resource");
//        }
        try {
            //resource = resourceDAO.get(path, conn);
            if (versionNumber == -1) {
                resource = resourceDAO.getLatestVersion(plainPath, conn);

            } else {
                resource = resourceDAO.get(plainPath, versionNumber, conn);
            }

        } catch (SQLException e) {

            String msg = Messages.getMessage("resource.get.fail", path);
            log.error(msg, e);
            throw new RegistryException(msg);

        } finally {
            try {
                conn.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }

        return resource;
    }

    public boolean put(String path, Resource resource) throws RegistryException {

        String userID = User.getCurrentUser();

        Connection conn = null;
        try {
            conn = dataSource.getConnection();
        } catch (SQLException e) {
            throw new RegistryException(e.getMessage());
        }

        try {
            conn.setAutoCommit(false);

            Resource a = resourceDAO.getLatestVersion(path, conn);
            if (a != null) {

                // given resource already exist. update it.
                resource.setId(a.getId());

                // if client has not explicitly set the author, keep the original author name.
                if (resource.getAuthorUserName() == null) {
                    resource.setAuthorUserName(a.getAuthorUserName());
                }

                resource.setState(RegistryConstants.ACTIVE_STATE);
                resource.setLastUpdaterUserName(userID);
                resourceDAO.update(path, resource, conn);
                resourceDAO.addResourceVersion(resource, conn);

            } else {

                // given resource does not exist. add it as a new resource.

                // first create all ascendant collections
                String[] parts = path.split(RegistryConstants.PATH_SEPARATOR);

                String currentPath = RegistryConstants.ROOT_PATH + parts[1];
                for (int i = 2; i < parts.length; i++) {

                    Resource currentCollection = resourceDAO.getLatestVersion(currentPath, conn);
                    if (currentCollection == null) {

                        currentCollection = new Resource();
                        currentCollection.setPath(currentPath);
                        currentCollection.setDirectory(true);
                        currentCollection.setAuthorUserName(userID);
                        currentCollection.setLastUpdaterUserName(userID);

                        resourceDAO.add(currentCollection, conn);
                        AuthorizationUtil.setDefaultAuthorizations(realm, currentPath, userID);
//                        currentCollection = resourceDAO.getLatestVersion(currentPath, conn);

                    } else {

                        if (currentCollection.getState() == RegistryConstants.DELETED_STATE) {

                            currentCollection = new Resource();
                            currentCollection.setPath(currentPath);
                            currentCollection.setDirectory(true);
                            currentCollection.setAuthorUserName(userID);
                            currentCollection.setLastUpdaterUserName(userID);
                            put(currentPath, currentCollection);

                        } else if (!currentCollection.isDirectory()) {

                            String message =
                                    Messages.getMessage("resource.adding.error", currentPath, path);
                            throw new RegistryException(message);
                        }
                    }

                    currentPath = currentPath + RegistryConstants.PATH_SEPARATOR + parts[i];
                }

                // now add the required resource
                resource.setPath(path);
                if (resource.getAuthorUserName() == null) {
                    resource.setAuthorUserName(userID);
                }
                resource.setLastUpdaterUserName(userID);

                resourceDAO.add(resource, conn);
                AuthorizationUtil.setDefaultAuthorizations(realm, path, userID);

                resource.setId(resourceDAO.getResourceID(path, conn));
                resourceDAO.addResourceVersion(resource, conn);

                // add new resource as a child of the current parent
                //resourceDAO.addChild(currentParent.getId(), resource.getId(), conn);
            }

            conn.commit();

        } catch (SQLException e) {

            try {
                conn.rollback();
            } catch (SQLException e1) {
                e1.printStackTrace();
            }

            String msg = Messages.getMessage("resource.put.fail", path, e.getMessage());
            log.error(msg, e);
            throw new RegistryException(msg);
        } finally {
            try {
                conn.close();
            } catch (SQLException e) {
                log.error("Exception occur in side the finaly block of " + this + " put");
            }
        }
        return true;
    }

    public boolean delete(String path) throws RegistryException {

        Connection conn = null;
        try {
            conn = dataSource.getConnection();
        } catch (SQLException e) {
            throw new RegistryException(e.getMessage());
        }

        try {
            conn.setAutoCommit(false);
            Resource resource = resourceDAO.getLatestVersion(path, conn);
            if (resource != null) {
                if (resource.isDirectory()) {
                    deleteDirectory(resource, conn);
                } else {
                    resourceDAO.markDeleted(path, conn);
                    resourceDAO.deleteResource(resource, conn);
                    //prepareToDelete(resource.getId(), conn);
                    //resourceDAO.delete(resource.getPath(), conn);
                }
            } else {
                String msg = Messages.getMessage("delete.null.resource", path);
                log.error(msg);
                throw new RegistryException(msg);
            }
            conn.commit();
        } catch (SQLException e) {
            try {
                conn.rollback();
            } catch (SQLException e1) {
                log.error("Exception occur in side the catch block of " + this + " delete");
            }
            String msg = Messages.getMessage("resource.delete.fail", path);
            log.error(msg, e);
            throw new RegistryException(msg);
        } finally {
            try {
                conn.close();
            } catch (SQLException e) {
                log.error("Exception occur in side the finaly block of " + this + " delete");
            }
        }
        return true;
    }

    private void deleteDirectory(Resource collection, Connection conn) throws SQLException {
        List childList = resourceDAO.getChildren(collection.getId(), -1, conn);
        Iterator i = childList.iterator();
        while (i.hasNext()) {
            Resource child = (Resource)i.next();
            if (child.isDirectory()) {
                deleteDirectory(child, conn);
            } else {
                resourceDAO.markDeleted(child.getPath(), conn);
                resourceDAO.deleteResource(child, conn);
            }
        }
        resourceDAO.markDeleted(collection.getPath(), conn);
        resourceDAO.deleteResource(collection, conn);
    }

    public boolean putChild(String childPath, Resource resource) throws RegistryException {
        // General resource should not have any constraint on adding children.
        return true;
    }
}
