/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.registry.servlet;

import org.apache.axiom.om.OMElement;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.Registry;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.RegistryException;
import org.wso2.registry.Resource;
import org.wso2.registry.app.RemoteRegistry;
import org.wso2.registry.config.DataBaseConfiguration;
import org.wso2.registry.config.RegistryConfiguration;
import org.wso2.registry.i18n.Messages;
import org.wso2.registry.jdbc.InMemoryJDBCRegistry;
import org.wso2.registry.jdbc.JDBCRegistry;
import org.wso2.registry.jdbc.realm.InMemoryRegistryRealm;
import org.wso2.registry.jdbc.realm.RegistryRealm;
import org.wso2.registry.jdbc.utils.RegistryDataSource;
import org.wso2.registry.secure.SecureRegistry;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URL;
import java.net.MalformedURLException;

/** Servlet for providing REST API for the registry. */
public class RegistryServlet extends HttpServlet {

    private static final Log log = LogFactory.getLog(RegistryServlet.class);
    protected transient ServletConfig servletConfig;

    //To store the context path
    private String contextRoot = null;
    private String configPath;
    private RegistryConfiguration registryConfiguration;

    public void init(ServletConfig config) throws ServletException {
        super.init(config);
        RegistryRealm registryRealm;
        Registry coreRegistry;
        try {
            // read the registry.xml file from the configured location. if not configured, read the
            // default registry.xml file bundled with the webapp.
            configPath = config.getInitParameter(RegistryConstants.REGISTRY_CONFIG_PATH);
            registryConfiguration = new RegistryConfiguration();
            registryConfiguration.populateRegistryConfig(configPath, config);
            config.getServletContext().
                    setAttribute(RegistryConstants.REGISTRY_CONFIG, registryConfiguration);

            DataBaseConfiguration dbConfiguration =
                    registryConfiguration.getDefaultDataBaseConfiguration();

            if (dbConfiguration.getConfigName().equals("in-memory")) {
                registryRealm = new InMemoryRegistryRealm();
                coreRegistry = new InMemoryJDBCRegistry(registryRealm);
            } else {
                DataSource dataSource;
                if (dbConfiguration.getDataSourceName() != null &&
                    dbConfiguration.getDataSourceName().length() > 0) {
                    try {
                        Context context = new InitialContext();
                        dataSource =
                                (DataSource)context.lookup(dbConfiguration.getDataSourceName());

                    } catch (NamingException e) {

                        String msg = "Could not find the specified data source named: " +
                                     dbConfiguration.getDataSourceName() + ". Caused by: " +
                                     e.getMessage();
                        log.fatal(msg, e);
                        throw new ServletException(msg);
                    }
                } else {
                    dataSource = new RegistryDataSource(
                            dbConfiguration.getDbUrl(),
                            dbConfiguration.getDriverName(),
                            dbConfiguration.getUserName(),
                            dbConfiguration.getPassWord());
                }

                registryRealm = new RegistryRealm(dataSource);
                coreRegistry = new JDBCRegistry(dataSource, registryRealm);
            }

            // create a system registry and put it in the context
            SecureRegistry systemRegistry =
                    new SecureRegistry(RegistryConstants.SYSTEM_USER,
                                       RegistryConstants.SYSTEM_PASSWORD,
                                       coreRegistry,
                                       registryRealm);

            try {
                RemoteRegistry remote = new RemoteRegistry(new URL("http://localhost:8080/wso2registry/atom"));
                config.getServletContext().setAttribute(RegistryConstants.REGISTRY, remote);
            } catch (MalformedURLException e) {
                config.getServletContext().setAttribute(RegistryConstants.REGISTRY, coreRegistry);
                e.printStackTrace();
            }
            config.getServletContext()
                    .setAttribute(RegistryConstants.REGISTRY_REALM, registryRealm);
            System.getProperties().put(RegistryConstants.REGISTRY, coreRegistry);
            System.getProperties().put(RegistryConstants.SYSTEM_REGISTRY, systemRegistry);
        } catch (RegistryException e) {
            throw new ServletException(e.getMessage());
        }

        log.info(Messages.getMessage("server.initalized"));
    }

    public void init() throws ServletException {
        if (servletConfig != null) {
            init(servletConfig);
        }
    }

    protected void doPost(HttpServletRequest httpServletRequest,
                          HttpServletResponse httpServletResponse)
            throws ServletException, IOException {
        initContextRoot(httpServletRequest);
        String uri = httpServletRequest.getRequestURI();
        String p1 = uri.substring(contextRoot.length(), uri.length());
        if (p1.equals("/web/addResource")) {
            handleFileUpload(httpServletRequest, httpServletResponse);
        }
    }

    private void handleFileUpload(HttpServletRequest request, HttpServletResponse response)
            throws IOException, ServletException {
        try {
            Registry registry = Utils.getSecureRegistry(request);
            Resource fileElement =
                    FileUploadUtil.processUpload(request);
            String path = fileElement.getPath();
            registry.put(path, fileElement);
            request.getSession().setAttribute(RegistryConstants.STATUS_MESSAGE_NAME,
                                              "Resource " + path +
                                              " is successfully added to the registry.");
            response.setContentType("text/html");
            request.getRequestDispatcher(RegistryConstants.RESOURCES_JSP)
                    .forward(request, response);

        } catch (RegistryException e) {

            e.printStackTrace();
            request.getSession().setAttribute(RegistryConstants.STATUS_MESSAGE_NAME,
                                              e.getMessage());
            response.setContentType("text/html");
            request.getRequestDispatcher(RegistryConstants.RESOURCES_JSP)
                    .forward(request, response);
        }
    }

    protected void doGet(HttpServletRequest request,
                         HttpServletResponse response)
            throws ServletException, IOException {
        initContextRoot(request);
    }

    /**
     * To find the name of the war , so getting that from the request context path
     *
     * @param req :  HttpServletRequest
     */
    public void initContextRoot(HttpServletRequest req) {
        if (contextRoot != null && contextRoot.trim().length() != 0) {
            return;
        }
        String contextPath = req.getContextPath();
        //handling ROOT scenario, for servlets in the default (root) context, this method returns ""
        if (contextPath != null && contextPath.length() == 0) {
            contextPath = "/";
        }
        this.contextRoot = contextPath;
    }

    protected void renderView(String jspName,
                              HttpServletRequest httpServletRequest,
                              HttpServletResponse httpServletResponse)
            throws IOException, ServletException {
        httpServletResponse.setContentType("text/html");
        httpServletRequest.getRequestDispatcher(jspName)
                .include(httpServletRequest, httpServletResponse);
    }


    public void destroy() {
        super.destroy();
        if (configPath == null) {
            configPath = getServletConfig().getServletContext().
                    getRealPath("/WEB-INF/registry.xml");
        }
        try {
            FileOutputStream fio = new FileOutputStream(configPath);
            OMElement registryXml = registryConfiguration.getRegistryConfigAsXML();
            registryXml.serialize(fio);
            fio.flush();
            fio.close();
        } catch (Exception e) {
            //
        }
    }
}
